//"Parcel Walk"
//Author: Benjamin C Watt (@feyleafgames)
//Made for Ludum Dare #22 December 16-19th, 2011
//The 48-hour Game Development Competition
/////////////////////////////////////////////////
// You spent so much time alone at the computer,
// when you went out into the world, you realized
// how much you still wanted to be alone
// despite all the annoying people who constantly
// approach. This game is about
// finding creative ways of getting them out of each level.
//////////////////////////////////////////////////
// 2D Platforming, smooth motion of characters
// using keyboard input for left/right movement,
// and jumping, also an action command to interact with the
// things on the level.
/////////////////////////////////////////////////
// Things on the level to use:
// Spring boards - insurance salesman
// Trap doors - rival
// Falling rocks - ex-girlfriend
// Dynamite - kitten (joke)
// Dimensional Vortex - postal worker
// Banana Peel - annoying neighbor
// Firehose Blast - elderly woman
////////////////////////////////////////////////

#include <SFML/System.hpp>
#include <SFML/Audio.hpp>
#include <SFML/Graphics.hpp>
#include <SFML/Window.hpp>
#include <iostream>
#include <fstream>
#include "mapelements.h"
#include "map.h"
#include "player.h"
#include "npc.h"
#include "props.h"
#include "sign.h"

bool isBoundsCollision(float x1, float y1)
{
	bool ret=false;
	if(x1+4<0 || y1+4<0 || x1+28>640 || y1+32>384) ret=true;
	return ret;
}

bool isMapCollision(float x1, float y1, Map& theMap)
{
	bool ret=false;
	sf::Vector2f chk(0,0);
	for(int y=0; y<12;y++)
	{
		for (int x=0;x<20;x++)
		{
			chk.x=float(x*32);
			chk.y=float(y*32);

			if((theMap.Block[x][y].isSolid) && !(x1+4>chk.x+32 || y1+4>chk.y+32 || x1+28<chk.x || y1+32<chk.y)) ret=true;
		}
	}
	return ret;
}

bool isNpcNearby(Player& p, NPC& npc)
{
	sf::Vector2f pmidpoint(p.pos.x+16, p.pos.y+16);
	sf::Vector2f npmidpoint(npc.pos.x+16, npc.pos.y+16);
	float dist=sqrt((pmidpoint.x-npmidpoint.x)*(pmidpoint.x-npmidpoint.x)+(pmidpoint.y-npmidpoint.y)*(pmidpoint.y-npmidpoint.y));
	if(dist<64) return true; else return false;
}
bool isSignNearby(Player& p, Sign& npc)
{
	sf::Vector2f pmidpoint(p.pos.x+16, p.pos.y+16);
	sf::Vector2f npmidpoint(npc.pos.x+16, npc.pos.y+16);
	float dist=sqrt((pmidpoint.x-npmidpoint.x)*(pmidpoint.x-npmidpoint.x)+(pmidpoint.y-npmidpoint.y)*(pmidpoint.y-npmidpoint.y));
	if(dist<32 && npc.isActive) return true; else return false;
}

bool isPropNearby(Player& p, Prop& item)
{
	sf::Vector2f pmidpoint(p.pos.x+16, p.pos.y+16);
	sf::Vector2f npmidpoint(item.pos.x+16, item.pos.y+16);
	float dist=sqrt((pmidpoint.x-npmidpoint.x)*(pmidpoint.x-npmidpoint.x)+(pmidpoint.y-npmidpoint.y)*(pmidpoint.y-npmidpoint.y));
	if(dist<64) return true; else return false;
}


int main(void)
{
	bool ending=false;
	sf::Clock storytime;
	storytime.Reset();
	sf::RenderWindow app(sf::VideoMode(640, 384, 32), "Parcel Walk by feyleaf :: LD22 48-hr Game");
	sf::Font smallfont;
	sf::View theView;
	smallfont.LoadFromFile("assets/minecraftia.ttf", 18);
	theView.SetHalfSize(320, 192);
	theView.SetCenter(320, 192);
	app.SetView(theView);
	app.SetFramerateLimit(120);
	sf::Event theEvent;
	sf::Sprite faderspr;
	faderspr.Resize(640, 384);
	sf::Uint8 alphafade=0;
	faderspr.SetColor(sf::Color(0, 0, 0, alphafade));
	const sf::Input &theInput=app.GetInput();
	sf::String pausewords("PAUSED", smallfont, 9);
	sf::String message("Arrows Move\n\'Space\' Jumps\n\'X\' or \'C\': Actions", smallfont, 9);
	message.SetColor(sf::Color(128, 96, 71, 255));
	message.SetPosition(12, 260);
	sf::SoundBuffer bufmusic;
	sf::SoundBuffer bufgush;
	sf::SoundBuffer bufspring;
	sf::SoundBuffer bufclick;
	sf::SoundBuffer bufding;
	bufgush.LoadFromFile("assets/gush.wav");
	bufclick.LoadFromFile("assets/click.wav");
	bufspring.LoadFromFile("assets/spring.wav");
	bufding.LoadFromFile("assets/ding.wav");
	bufmusic.LoadFromFile("assets/ilikealone.ogg");
	sf::Sound gush(bufgush);
	sf::Sound click(bufclick);
	sf::Sound spring(bufspring);
	sf::Sound ding(bufding);
	gush.SetLoop(false);
	sf::Sound music(bufmusic);
	sf::Image spritesheet;
	Map map;
	int level=1;
	map.LoadFromFile("assets/level1.txt", level);
	spritesheet.SetSmooth(false);
	spritesheet.LoadFromFile("assets/sprites.png");
	Player player;
	NPC oldlady;
	NPC kitten;
	Prop spray[2];
	Sign sign;
	bool levtrans=false;
	bool story=true;
	bool talked=false;
	bool catyes=true;
	Prop hydrant;
	sign.SetGraphics(spritesheet, 7*32, 2*32);
	sign.pos=sf::Vector2f(608, 200);
	sign.isActive=false;
	oldlady.Initialize(map.npcset, map.npcpos, spritesheet);
	kitten.Initialize(5, map.catpos, spritesheet);
	hydrant.Initialize(map.propset, map.proppos, spritesheet);
	spray[0].Initialize(3, map.proppos, spritesheet);
	spray[1].Initialize(3, map.proppos, spritesheet);

		while(story && storytime.GetElapsedTime()<4)
		{
			app.GetEvent(theEvent);
			if(theEvent.Type==sf::Event::KeyPressed) story=false;
			app.Clear();
			pausewords.SetSize(30);
			pausewords.SetText("    Parcel Walk\n\n\n\n Ludum Dare 22\n Theme: 'Alone'\n  by: feyleaf\n(Benjamin C Watt)");
			pausewords.SetColor(sf::Color::White);
			pausewords.SetPosition(100, 75);
			app.Draw(pausewords);
			app.Display();
		}
		while(story && storytime.GetElapsedTime()<9)
		{
			app.GetEvent(theEvent);
			if(theEvent.Type==sf::Event::KeyPressed) story=false;
			app.Clear();
			pausewords.SetSize(24);
			pausewords.SetText("\nRobby likes to be alone.\nHe spends time\nalone at the computer\nmost days.\n\nToday, though, he\ngot a delivery note\nsaying he had\na new computer waiting for him\nat the Post Office.\n\npress a key to begin...");
			pausewords.SetColor(sf::Color::White);
			pausewords.SetPosition(0,0);
			app.Draw(pausewords);
			app.Display();
		}
		pausewords.SetText("PAUSED");

	player.SetGraphics(spritesheet, 0, 0);
	pausewords.SetPosition(40, 260);
	bool quit=false;
	bool paused=false;

	music.SetLoop(true);
	if(music.GetStatus()==sf::Sound::Stopped) music.Play();
	theView.SetHalfSize(160, 96); //zoom in 2x
	while(!quit)
	{

		app.GetEvent(theEvent);
		if(theEvent.Type==sf::Event::Closed) quit=true;
		if(theEvent.Type==sf::Event::LostFocus || theEvent.Type==sf::Event::Resized) paused=true;
		if(theEvent.Type==sf::Event::GainedFocus || theEvent.Type==sf::Event::MouseButtonPressed) paused=false;

		if(!levtrans && !ending)
		{
			if(theInput.IsKeyDown(sf::Key::Down))
			{
				//player.vel+=sf::Vector2f(0,0.21f);
			}
			if(theInput.IsKeyDown(sf::Key::Space))
			{
				player.Jump(-4.6f);
			}
			if(theInput.IsKeyDown(sf::Key::Right))
			{
				player.isLeft=false;
				player.vel+=sf::Vector2f(0.125f, 0);
			}
			if(theInput.IsKeyDown(sf::Key::Left))
			{
				player.isLeft=true;
				player.vel+=sf::Vector2f(-0.125f, 0);
			}
			if(isNpcNearby(player, oldlady) && !sign.isActive)
			{
				//talk to the npc
				talked=true;
				message.SetColor(sf::Color::Black);
				message.SetText(oldlady.speech.GetText());
				message.SetPosition(oldlady.pos.x-60, oldlady.pos.y-16);
			}
			if(isNpcNearby(player, kitten))
			{
					//talk to the kitten
				if(catyes) {ding.Play(); catyes=false;}
				message.SetColor(sf::Color::Blue);
				message.SetText(kitten.speech.GetText());
				message.SetPosition(kitten.pos.x-60, kitten.pos.y-16);
			}
			if((theInput.IsKeyDown(sf::Key::X) || theInput.IsKeyDown(sf::Key::C)) && !sign.isActive && isPropNearby(player, hydrant))
			{
				if(hydrant.proptype==0)
				{
					//use the fire hydrant
					message.SetColor(sf::Color::Red);
					message.SetText(player.speech.GetText());
					message.SetPosition(player.pos.x-60, player.pos.y-16);
					gush.Play();
					spray[0].isActive=true;
					spray[0].vel=sf::Vector2f(-5.5, -0.5);
					spray[1].isActive=true;
					spray[1].vel=sf::Vector2f(-4.5, -0.5);

					hydrant.SetGraphics(spritesheet, 160, 32);
					oldlady.vel+=sf::Vector2f(-3.5, -2.5);
					sign.isActive=true;
				}
				else if(hydrant.proptype==1)
				{
					//use the trapdoor
					message.SetColor(sf::Color::Red);
					message.SetText(player.speech.GetText());
					message.SetPosition(player.pos.x-60, player.pos.y-16);
					hydrant.spr.Rotate(-2.5f);
					if(hydrant.spr.GetRotation()<280)
					{
						oldlady.isFall=true;
						if(oldlady.index==1)
							oldlady.SetGraphics(spritesheet, 6*32, 32);
						click.Play();
						map.Block[int((hydrant.pos.x+16)/32)][int((hydrant.pos.y+16)/32)].isSolid=false;
						sign.isActive=true;
					}
				}
				else if(hydrant.proptype==2)
				{
					//use the spring board
					message.SetColor(sf::Color::Red);
					message.SetText(player.speech.GetText());
					message.SetPosition(player.pos.x-60, player.pos.y-16);
					spring.Play();
					hydrant.SetGraphics(spritesheet, 96, 96);
					oldlady.vel=sf::Vector2f(1.5, -28.5);
					sign.isActive=true;
				}
			}
		}
		else
		{
			if(!ending)
			{
			//beat the level
			alphafade+=4;
			if(alphafade>250)
			{
				alphafade=0;
				levtrans=false;
				level++; if(level>5) ending=true;
				else{
				map.LoadFromFile("assets/level1.txt", level);
				talked=false;
				player.Initialize();
				catyes=true;
				message.SetText("");
				sign.Initialize((level==5), sf::Vector2f(18*32, 6*32), spritesheet);
				kitten.Initialize(5, map.catpos, spritesheet);
				oldlady.Initialize(map.npcset, map.npcpos, spritesheet);
				hydrant.Initialize(map.propset , map.proppos, spritesheet);
	spray[0].Initialize(3, map.proppos, spritesheet);
	spray[1].Initialize(3, map.proppos, spritesheet);
				}
			}
			faderspr.SetColor(sf::Color(0, 0, 0, alphafade));
			}
		}

		if(isSignNearby(player, sign) && !levtrans)
		{
			levtrans=true;
			ding.Play();
		}

		kitten.Update(map);
		player.Update(map);
		oldlady.Update(map);
		hydrant.Update(map);
		spray[0].Update(map);
		spray[1].Update(map);
		if(sign.isActive) sign.Update(map);



		
		app.Clear();

		if(!paused)
		{
			if(!ending){
				
			if(music.GetStatus()==1) music.Play();
			map.Render(app);
			hydrant.Render(app);
			if(spray[0].isActive) spray[0].Render(app);
			if(spray[1].isActive) spray[1].Render(app);
			oldlady.Render(app);
			kitten.Render(app);
			if(sign.isActive) sign.Render(app);
			player.Render(app);
			app.Draw(message);
			app.Draw(faderspr);
			}else
			{
				if(ending){
				message.SetPosition(120, 16);
			message.SetColor(sf::Color::White);
			message.SetSize(30);
			message.SetText("So Robby signed for\nhis new computer at\nthe Post Office.\n\nHowever, since he\nis all alone,\nthere is no one to\nhelp him carry it\nhome.\n\nThe End.");
			theView.SetCenter(320, 186);
			theView.SetHalfSize(320, 186);
			music.Stop();
			 app.Draw(message);
			 if(theEvent.Type==sf::Event::KeyPressed) quit=true;
			}
			}
		}
		else if(!ending)
		{
			app.Draw(pausewords);
			player.Render(app);
			if(music.GetStatus()!=1) music.Pause();
		}


///set the camera///
		if(player.pos.y>256) //static view on y-axis
		{
			if(player.pos.x<192) //static view on left-side of x-axis
			{
				theView.SetCenter(160, 288);
			}
			else if(player.pos.x>512) //static view on right-side of x-axis
			{
				theView.SetCenter(480, 288);
			}
			else //variable on x-axis
				theView.SetCenter(player.pos.x-32, 288);
		}
		else //variable on y-axis
		{
			if(player.pos.x<192) //static view on left-side of x-axis
			{
				theView.SetCenter(160, player.pos.y+32);
			}
			else if(player.pos.x>512)
			{
				theView.SetCenter(480, player.pos.y+32);
			}
			else
				theView.SetCenter(player.pos.x-32, player.pos.y+32);
		}
		app.SetView(theView);

///show the renders
		app.Display();
	}
	return 1;
}
