//npc.h
#include <SFML/Graphics.hpp>
bool isBoundsCollision(float x, float y);
bool isMapCollision(float x, float y, Map& theMap);


class NPC
{
public:
	int ani, anitime;
	int index;
	sf::Vector2f pos;
	sf::Vector2f vel;
	sf::Sprite spr;
	bool isJump;
	bool isFall;
	bool isAnim;
	int jumptime;
	int maxjump;
	sf::Vector2f sel;
	sf::String speech;

	NPC();
	void Initialize(int ix, sf::Vector2f pox, sf::Image& img);
	void SetGraphics(sf::Image& img, int x, int y);
	void Render(sf::RenderWindow& win);
	void Update(Map& theMap);

};

NPC::NPC()
{
	index=0;
	ani=0;
	anitime=0;
	pos.x=0.0f;
	pos.y=0.0f;
	vel.x=0.0f;
	vel.y=0.0f;
	sel.x=0; sel.y=0;
	isJump=false;
	isFall=true;
	isAnim=true;
	maxjump=6;
	jumptime=0;
	spr.SetColor(sf::Color::White);
	speech.SetText("I am Error.");
}

void NPC::SetGraphics(sf::Image& img, int x, int y)
{
	spr.SetImage(img);
	spr.SetSubRect(sf::IntRect(x, y, x+32, y+32));
}

void NPC::Update(Map& theMap)
{
//apply brakes for speed bounds
	if(vel.x>0)
	{
		if(vel.x<0.15f) vel.x=0;
		else vel.x-=0.05f;
	}

//apply brakes for speed bounds
	if(vel.x<0)
	{
		if(vel.x>-0.15f) vel.x=0;
		else vel.x+=0.05f;
	}
//accelerate
	pos.x+=vel.x;

	if(isMapCollision(pos.x, pos.y, theMap))
	{
		//hit a wall and stop
		pos.x-=vel.x;
		vel.x=0;
	}

	if(!isJump || isFall)
		vel.y+=0.15f;
	else if(isJump)
	{
		isFall=true;
		jumptime++;
		if(jumptime>=maxjump)
			isFall=true;
	}
//terminal velocity
	if(vel.y>6.6f) vel.y=6.6f;
//more acceleration
	pos.y+=vel.y;
	if(isMapCollision(pos.x, pos.y, theMap))
	{
		if(vel.y>0)
		{
//falling
			pos.y-=vel.y;
			vel.y=0;
			isJump=false;
			isFall=false;
			jumptime=0;
		}
		else if(vel.y<0)
		{
//rising
			pos.y-=vel.y;
			vel.y=0;
			isJump=false;
			isFall=false;
			jumptime=maxjump;
		}
	}
		if(!isJump && !isFall && isAnim)
		{
			spr.SetSubRect(sf::IntRect(sel.x, (ani*32)+sel.y, sel.x+32, sel.y+(ani*32)+32));
			anitime++;
			if(anitime>56)
			{
				ani++;
				if(ani>1) ani=0;
				anitime=0;
			}
		}
}

void NPC::Render(sf::RenderWindow& win)
{
	spr.SetPosition(pos);
	win.Draw(spr);
}

void NPC::Initialize(int ix, sf::Vector2f pox, sf::Image& img)
{
	index=ix;
	ani=0;
	anitime=0;
	pos.x=pox.x;
	pos.y=pox.y-2;
	vel.x=0.0f;
	vel.y=0.0f;
	isJump=false;
	isFall=true;
	isAnim=true;
	maxjump=6;
	jumptime=0;
	switch(ix)
	{
	case 0: //old lady
	speech.SetText("Can you help me cross\nthe street?.");
		sel.x=4*32;
		sel.y=0;
		SetGraphics(img, sel.x, sel.y);
		break;
	case 1: //insurance salesman
	speech.SetText("I can offer you a GREAT\ndeal on life insurance!");
		sel.x=7*32;
		sel.y=0;
		SetGraphics(img, sel.x, sel.y);
		break;
	case 2: //rival
	speech.SetText("Heh. You'll never become as cool\nas me.");
		sel.x=5*32;
		sel.y=2*32;
		SetGraphics(img, sel.x, sel.y);
		break;
	case 3: //ex-girlfriend
//	speech.SetText("You never call anymore!\nIt's that computer...");
	speech.SetText("You never call since you\nentered Ludum Dare...");
		sel.x=6*32;
		sel.y=2*32;
		SetGraphics(img, sel.x, sel.y);
		break;
	case 4:
		//po worker - no anim!
	speech.SetText("I have your package, sir.");
		sel.x=32;
		sel.y=3*32;
		SetGraphics(img, sel.x, sel.y);
		isAnim=false;
		break;
	case 5:
	default:
		//kitten
	speech.SetText("            =^.^=");
		sel.x=4*32;
		sel.y=3*32;
		SetGraphics(img, sel.x, sel.y);
		isAnim=false;
		break;
	}
}
