//player.h
#include <SFML/Graphics.hpp>
bool isBoundsCollision(float x, float y);
bool isMapCollision(float x, float y, Map& theMap);


class Player
{
public:
	sf::Vector2f pos;
	sf::Vector2f vel;
	sf::Sprite spr;
	bool isJump;
	bool isFall;
	bool isLeft;
	int jumptime;
	int maxjump;
	int ani;
	int anitime;
	sf::String speech;

	Player();
	void Jump(float speed);
	void SetGraphics(sf::Image& img, int x, int y);
	void Render(sf::RenderWindow& win);
	void Update(Map& theMap);
	void Initialize();

};

Player::Player()
{
	anitime=0;
	ani=0;
	pos.x=0.0f;
	pos.y=0.0f;
	vel.x=0.0f;
	vel.y=0.0f;
	isJump=false;
	isFall=true;
	maxjump=6;
	jumptime=0;
	spr.SetColor(sf::Color::White);
	speech.SetText("I just want to be alone!");
}

void Player::Initialize()
{
	anitime=0;
	ani=0;
	pos.x=0.0f;
	pos.y=0.0f;
	vel.x=0.0f;
	vel.y=0.0f;
	isJump=false;
	isFall=true;
	maxjump=6;
	jumptime=0;
}

void Player::SetGraphics(sf::Image& img, int x, int y)
{
	spr.SetImage(img);
	spr.SetSubRect(sf::IntRect(x, y, x+32, y+32));
}

void Player::Update(Map& theMap)
{
//apply brakes for speed bounds
	if(vel.x>0)
	{
		if(vel.x<0.1f) vel.x=0;
		else vel.x-=0.1f;
	}
	if(vel.x>2) vel.x=2;


//apply brakes for speed bounds
	if(vel.x<0)
	{
		if(vel.x>-0.1f) vel.x=0;
		else vel.x+=0.1f;
	}
	if(vel.x<-2) vel.x=-2;

//accelerate
	pos.x+=vel.x;

	if(isMapCollision(pos.x, pos.y, theMap) || isBoundsCollision(pos.x, pos.y))
	{
		//hit a wall and stop
		pos.x-=vel.x;
		vel.x=0;
	}

	if(!isJump || isFall)
		vel.y+=0.15f;
	else if(isJump)
	{
		isFall=true;
		jumptime++;
		if(jumptime>=maxjump)
			isFall=true;
	}
//terminal velocity
	if(vel.y>6.6f) vel.y=6.6f;
//more acceleration
	pos.y+=vel.y;
	if(isMapCollision(pos.x, pos.y, theMap) || isBoundsCollision(pos.x, pos.y))
	{
		if(vel.y>0)
		{
//falling
			pos.y-=vel.y;
			vel.y=0;
			isJump=false;
			isFall=false;
			jumptime=0;
		}
		else if(vel.y<0)
		{
//rising
			pos.y-=vel.y;
			vel.y=0;
			isJump=false;
			isFall=false;
			jumptime=maxjump;
		}
	}
	if(vel.x>0.15f || vel.x<-0.15f)
	{
		spr.SetSubRect(sf::IntRect(0, ani*32, 32, ani*32+32));
		anitime++;
		if(anitime>12)
		{
			ani++;
			if(ani>3) ani=2;
			anitime=0;
		}
	}
	else if(!isJump && !isFall)
	{
		spr.SetSubRect(sf::IntRect(0, ani*32, 32, ani*32+32));
		anitime++;
		if(anitime>56)
		{
			ani++;
			if(ani>1) ani=0;
			anitime=0;
		}
	}

}

void Player::Jump(float speed)
{
	if(jumptime==0) //allow a jump to start
	{
		if(!isJump)
		{
			isJump=true;
			vel.y+=speed;
		}
		else
		{
			if(jumptime<maxjump && !isFall)
			{
				vel.y+=speed;
			}
		}
	}
}

void Player::Render(sf::RenderWindow& win)
{
	spr.SetPosition(pos);
	spr.FlipX(isLeft);
	win.Draw(spr);
}

