//sign.h
#include <SFML/Graphics.hpp>
bool isBoundsCollision(float x, float y);
bool isMapCollision(float x, float y, Map& theMap);


class Sign
{
public:
	sf::Vector2f pos;
	sf::Vector2f vel;
	sf::Sprite spr;
	bool isJump;
	bool isFall;
	int jumptime;
	int maxjump;
	bool isActive;
	bool isLeft;

	Sign();
	void Initialize(bool isL, sf::Vector2f pox, sf::Image& img);
	void SetGraphics(sf::Image& img, int x, int y);
	void Render(sf::RenderWindow& win);
	void Update(Map& theMap);

};

Sign::Sign()
{
	pos.x=0.0f;
	pos.y=0.0f;
	vel.x=0.0f;
	vel.y=0.0f;
	isJump=false;
	isFall=true;
	isActive=false;
	isLeft=true;
	maxjump=6;
	jumptime=0;
	spr.SetColor(sf::Color::White);
}

void Sign::Initialize(bool isL, sf::Vector2f pox, sf::Image& img)
{
	pos.x=pox.x;
	pos.y=pox.y-2;
	vel.x=0.0f;
	vel.y=0.0f;
	isJump=false;
	isFall=true;
	isActive=false;
	isLeft=isL;
	maxjump=6;
	jumptime=0;
	if(isL) SetGraphics(img, 7*32, 3*32);
	else SetGraphics(img, 7*32, 2*32);
}

void Sign::SetGraphics(sf::Image& img, int x, int y)
{
	spr.SetImage(img);
	spr.SetSubRect(sf::IntRect(x, y, x+32, y+32));
}

void Sign::Update(Map& theMap)
{
//apply brakes for speed bounds
	if(vel.x>0)
	{
		if(vel.x<0.15f) vel.x=0;
		else vel.x-=0.15f;
	}

//apply brakes for speed bounds
	if(vel.x<0)
	{
		if(vel.x>-0.15f) vel.x=0;
		else vel.x+=0.15f;
	}
//accelerate
	pos.x+=vel.x;

	if(isMapCollision(pos.x, pos.y, theMap) || isBoundsCollision(pos.x, pos.y))
	{
		//hit a wall and stop
		pos.x-=vel.x;
		vel.x=0;
	}

	if(!isJump || isFall)
		vel.y+=0.15f;
	else if(isJump)
	{
		isFall=true;
		jumptime++;
		if(jumptime>=maxjump)
			isFall=true;
	}
//terminal velocity
	if(vel.y>6.6f) vel.y=6.6f;
//more acceleration
	pos.y+=vel.y;
	if(isMapCollision(pos.x, pos.y, theMap) || isBoundsCollision(pos.x, pos.y))
	{
		if(vel.y>0)
		{
//falling
			pos.y-=vel.y;
			vel.y=0;
			isJump=false;
			isFall=false;
			jumptime=0;
		}
		else if(vel.y<0)
		{
//rising
			pos.y-=vel.y;
			vel.y=0;
			isJump=false;
			isFall=false;
			jumptime=maxjump;
		}
	}

}

void Sign::Render(sf::RenderWindow& win)
{
	spr.SetPosition(pos);
	win.Draw(spr);
}

